/**************************************************************************
                            PROYECTO MANTA
    DERECHOS RESERVADOS A LA UNIVERSIDAD DE LOS ANDES  (c) 1985-1997
 **************************************************************************
                       API PARA EL KERNEL DE MANTA
Descripcion: Contiene funciones para accesar el kernel de ManTa de forma
             ortogonal
Autor del modulo:   Vladimir Tamara Patino
Version del modulo : 1.0
Version del Kernel: 3.1
              Santafe de Bogota D.C. (COLOMBIA)  May-Jun / 1997
 **************************************************************************/

#include "definic.h"

/* @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
                           DEL API PARA EL API
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ */

/* Inicializacion del API */
extern int iniMantaAPI(char *nom);

/* Finalizacion API */
extern int finMantaAPI(void);

/* @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
                                 TAD TIPO
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ */

/* Indica si el tipo dado es basico */
int tipoBasico(nodo_tad *);


/* Inica si el nombre de un tipo es valido */
int nombreTipoValido(char *,int,char *);

/* Carga un tipo del archivo abierto que recibe */
extern int cargaT(FILE *,nodo_tad **,char *);


/* Carga del archivo tipo texto abierto que recibe un TAD
(hace analisis sintactico) y llena con este el tad que recibe */
extern int cargaArchTextoT(FILE *, nodo_tad **, char *);

/* Crea una archivo con la representacion del TAD que recibe */
extern void creaArchTextoT( FILE *, nodo_tad *);

extern const int CP_ENC,   /* Generar Encabezados */
                 CP_CON,   /* Generar Constructoras */
                 CP_SEL,   /* Generar Selectoras */
                 CP_INT,   /* Generar Interfaz */
                 CP_TODO;  /* Generar Todo */


/* Elimina una lista de nombres */
int elimNodoNombre(nodo_nombre **n);

/* Elimina una expresion */
int elimNodoExpresion(nodo_expresion **n);

/* Elimina una lista de argumentos */
int elimNodoArgumento(nodo_argum **n);

/* Elimina una lista de Igualdades */
extern int elimNodoIgualdad(nodo_igualdad **);

/* Elimina una lista de generadoras */
int elimNodoGeneradora(nodo_generadora **n);

/* Saca una copia de un TAD */
int copiaT(nodo_tad **dest,nodo_tad *fte,char *merr);

/* Elimina de memoria el TAD que recibe */
extern int eliminaT(nodo_tad **);

/*  Genera los archivos del prototipo en C del TAD que recibe */
extern int generaCProt(nodo_tad *t,int que);

/* Retorna el nombre del TAD que recibe */
extern void nombreT(nodo_tad *, int, char *);

/* Crea un nuevo nodo de un tipo, por instanciacin */
extern int nuevoInstanciacionT(char *,nodo_tad *,
           char *,nodo_tad *, nodo_tad **,char *merr);

/* Crea un nuevo nodo de un tipo, por copia de otro*/
extern int nuevoCopiaT(char *, nodo_tad *, nodo_tad **,char *);

/* Crea un nuevo nodo de un tipo como producto de otros dos */
extern int nuevoProdT(char *, nodo_tad *, nodo_tad *, nodo_tad **,char *);

/*   Genera los axiomas para la selectora sel, sabiendo que es una selectora
     de un tipo que restringe otro */
int genereAxiomaRestriccion(char *tr,nodo_select *sel,char *npred, char *merr);

/* Cambia todos los parametros de la selectora sel a inductivos o no 
  segun el parametro ind */
void cambiaParametrosInduccion(nodo_select *sel, short ind);

/*Crea un nuevo nodo de un tipo como restriccion de otro por un predicado */
extern int nuevoResT(char *, nodo_tad *, char *, char *,
              FILE *,nodo_tad **,char *);

/*Crea un nuevo nodo de un tipo como suma directa de dos existentes */
extern
int nuevoSumT(char *, nodo_tad *, nodo_tad *,FILE *,nodo_tad **,char *);

/*Crea un nuevo nodo de un tipo como completacion de otro con un elemento */
extern
int nuevoComT(char *, nodo_tad *, char *,FILE *,nodo_tad **,char *);

/*Crea un nuevo nodo de un tipo como variacion de dos existentes */
extern
int nuevoVarT(char *, nodo_tad *, nodo_tad *,
              FILE *,nodo_tad **,char *);

/*Crea un nuevo nodo de un tipo inductivo */
extern int nuevoIndT(char *, FILE *,nodo_tad **,char *);

/* Crea un nuevo nodo de un tipo */
extern int nuevoT(nodo_tad **,char *merr);

/* Retorna la cantidad de parametros del tad que recibe */
extern int numParamT(nodo_tad *);

/* Retorna un parametro del tad recibido */
extern void nombreParamT(nodo_tad *,int ,char *);

/* Salva el tipo que recibe en el archivo abierto que recibe */
extern int salvaT(FILE *fp,nodo_tad *nt);

/* Compone 2 representaciones para producir una nueva */
int componeRep(char *nom,nodo_tad *r1, nodo_tad *r2,
	       nodo_tad **nuevo,char *merr);

/* Crea la definicion parcial de una representacion de t1 con t2 */
int nuevaRep(char *nom,nodo_tad *t1, nodo_tad *t2,
	     FILE *fp,nodo_tad **nuevo,char *merr);


/* Crea las conjeturas necesarias para probar una representacion */
int conjeturasRep(FILE *fp,nodo_tad *rep,char *merr);

/* Analiza cambios a una representacion y la actualiza */
int cambiosRep(nodo_tad **rep,FILE *fp,char *merr);

/**** PARA DEPURAR ***/
#ifndef NDEBUG
/* Muestra la informacion del Tipo que recibe */
void muestraT(nodo_tad *);
#endif

/* @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
                             TAD LISTA DE TIPOS
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ */

/* CONSTANTES PARA CONFIGURAR ADICION */
extern const int ADIC_ADD, /* Siempre agrega */
          ADIC_REP;        /* Reemplaza */

/* Agrega en orden alfabetico el elemento nuevo */
int adicElemLT(nodo_tad **,nodo_tad *, int,char *);

/* Retorna la cantidad de Tipos en la lista iniciada con el tad que recibe */
extern int cantElemLT(nodo_tad *);

/* Si existe un tipo con el nombre recibido lo elimina */
void elimElemLT(nodo_tad **tad,char *nom);

/* Retorna el nombre del i-esimo Tipo de la lista que recibe */
extern void nombreElemLT(nodo_tad *,int, char *);

/* Retorna un apuntador al TAD cuyo nombre es el recibido */
extern nodo_tad *infoBuscaNombreLT(nodo_tad *,char *);

/**** PARA DEPURAR ***/
#ifndef NDEBUG
/* Muestra la lista completa */
void muestraLT(nodo_tad *);
#endif



/* @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
                               MANEJO DE ERRORES
   @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ */

#define MAX_MENS_RET 3000

extern char *mensaje_retornado;
  /* Mensaje retornado con errores o informacin */
extern int cod_ult_mensaje;
  /* Codigo del ultimo mensaje retornado */

/* Dice si hay o no mensajes pendientes */
extern int hayMensajes(void);

/* Retorna el mensaje ms reciente que se tiene almacenado */
extern void mensajeReciente(char *m);

/* Borra buffer de mensajes por retornar */
extern void borraMensajes(void);

/* @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
                               UTILIDADES
   @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ */

/* Dice si el nombre que recibe es valido para un TAD
   o nombre de funcion */
int esNombreValido(char *nom);

