/* Test for ManTa Kernel (3.1 and 3.2) */
/* By Vladimir Tamara P. */
/* 1998 */

#include <stdio.h>
#include "definic.h"
#include "apimanta.h"
#include "os_dep.h"
#include <assert.h>
#include <stdlib.h>

/***************************** PROTOTYPES *******************************/

/* Prints the contents of a file in stdout */
int imprimeArchivo(char *aname);

/* Loadas a type written in a text file (not in environment) */
nodo_tad *loadTextType(char *name, char *desc);

/* Prints an ADT */
int imprimeT(nodo_tad *nt,char *inf);

/* This function finds the normal form of the expression it receives */
int normaliza(char *expr);

int testNormalization(void);

int testTheorems(void);

int testPrototypes(void);

int testCopy(nodo_tad *nat,nodo_tad *TBool,nodo_tad *cola,nodo_tad *nogen);

int testProduct(nodo_tad *nat,nodo_tad *TBool, nodo_tad *cola,nodo_tad *nogen);

int testRestriction(nodo_tad *nat,nodo_tad *TBool, nodo_tad *cola,
		    nodo_tad *nogen);

int testDirectSum(nodo_tad *nat,nodo_tad *TBool,nodo_tad *cola,nodo_tad *nogen);

int testCompletation(nodo_tad *nat,nodo_tad *TBool,
		     nodo_tad *cola,nodo_tad *nogen);

int testVariation(nodo_tad *nat,nodo_tad *TBool,nodo_tad *cola, 
		  nodo_tad *nogen);

int testInstance(nodo_tad *nat,nodo_tad *TBool,nodo_tad *cola,nodo_tad *nogen);

int testAxioms(void);

int testSignature(void);

int testGrammar(void);

int testRepresentations(nodo_tad *nat, nodo_tad *TBool,nodo_tad *cola,
			nodo_tad *nogen);


/*****************************    MAIN    *******************************/

int main()
{
  
  nodo_tad *nat,*cola,*TBool,*nogen;
  char merr[2000]; /* Error messages */
  char arcnam[500];
  FILE *fp;

  printf("# Test for  ManTa's Kernel 3.1\n");
  printf("# The testing environment is:\n");
  printf("#  - The Manta directory structure \n"
	 "#  - A manta.ini, configuring everything in english\n"
	 "#  - The test files (*.mta) \n");
  printf("# -----------------------------------------------------\n\n");

  printf("Initializing kernel\n");
  if (iniMantaAPI("manta.ini")<0)
    {
      printf("Failed (sure you hava manta.ini??)\n");
      return 1;
    }
  printf("Succesfull initialization\n");
  printf("Using NAT\n");    
  if ((nat=infoBuscaNombreLT(tad,"NAT"))==NULL)
    {
      printf("Failed. (Sure you have directory with types??) \n");
      return 1;
    }
  
  printf("Using bool\n");
  if ((TBool=infoBuscaNombreLT(tad,"BOOL"))==NULL)
    {
      printf("Failed. (Sure you have directory with types??) \n");
      return 1;
    }
  printf("Loading cola\n");
  sprintf(arcnam,"%s/cola.adt",direct[2]);
  printf("Opening %s\n",arcnam);
  fp=fopen(arcnam,"rt");
  if (fp==NULL)
    {  printf("File not found %s\n",arcnam);  }
  else
    {
      merr[0]='\0';
      cola=NULL;
      if (cargaT(fp,&cola,merr)!=0)
	{
	  printf("Can't load type %s\n",arcnam); 
	  return 1;
	}
      else
	{  imprimeT(cola,"TAD Cola"); }
      fclose(fp);
    }

  printf("Loading type withot generators (nogen1)\n");
  nogen=loadTextType("nogen1.mta","No generators");
  if (nogen==NULL || adicElemLT(&tad,nogen,ADIC_REP,merr)!=0)
    {
      printf("Error adding type nogen1\n");
      return 1;
    }

  printf("\n# I. TESTING TYPES\n");

  printf("# 1. Testing Grammar\n");
  testGrammar();
  testSignature();
  testAxioms();
  printf("# 2. Testing definition of new types\n");
  testInstance(nat,TBool,cola,nogen);
  testCopy(nat,TBool,cola,nogen);
  testProduct(nat,TBool,cola,nogen);
  testRestriction(nat,TBool,cola,nogen);
  testDirectSum(nat,TBool,cola,nogen);
  testCompletation(nat,TBool,cola,nogen);
  testVariation(nat,TBool,cola,nogen);
  testRepresentations(nat,TBool,cola,nogen);
  
  printf("\n# II. TESTING THEOREMS AND NORMALIZATION\n");
  testNormalization();
  testTheorems();

  printf("\n# III. TESTING PROTOTYPES\n");
  testPrototypes();
  return finMantaAPI();
}


/***************************** FUNCTIONS DEFINITION *************************/

/* Prints the contents of a file in stdout */
int imprimeArchivo(char *aname)
{
  FILE *fp;
  char lin[1000];

  printf("Printing file: %s\n",aname);
  fp=fopen(aname,"rt");
  if (fp==NULL)
    {
      printf("Couldn't read from %s\n",aname);
      return -1;
    }
  while (!feof(fp))
    {
      fgets(lin,1000,fp);
      printf("%s",lin);
    }
  fclose(fp);

  return 0;
}

/* Loadas a type written in a text file (not in environment) */
nodo_tad *loadTextType(char *name, char *desc)
{

  FILE *f;
  texpr msg;
  nodo_tad *tad=NULL; 
  assert(name!=NULL);

  printf("# File:%s: %s\n",name,desc);
  f=fopen(name,"rb");
  if (f==NULL)
    {
      printf("File not found. \n");
      return NULL;
    }
  msg[0]='\0';
  if (cargaArchTextoT(f,&tad,msg))
    {
      printf("Error loading file. Message:\n");
      printf("%s\n",msg);
    }
  fclose(f);
  return tad;
}



/* Prints an ADT */
int imprimeT(nodo_tad *nt,char *inf)
{

  FILE *fp;
  char aname[1000];

  assert(nt!=NULL);
  sprintf(aname,"%s.mta",nt->nombre);
  printf("# Writing file: %s.   %s\n",aname,inf);
  fp=fopen(aname,"wt");
  if (fp==NULL)
    {
      printf("Couldn't write file %s\n",aname);
      return -1;
    }
  creaArchTextoT(fp,nt);
  fclose(fp);
  imprimeArchivo(aname);
  loadTextType(aname,"Testing generated type");
  return 0;
}

/* This function finds the normal form of the expression it receives */
int normaliza(char *expr)
{
  FILE *fp;
  nodo_argum *arg = NULL;
  nodo_igualdad *conj = NULL;
  texpr mens_error;

  fp=fopen("normaliz","wt");
  if (fp==NULL)
    {
      printf("Couldn't write file normaliz\n");
      return -1;
    }
  fprintf(fp,"%s",expr);
  fclose(fp);
  fp=fopen("normaliz","rt");
  if (fp==NULL)
    {
      printf("Couldn't read file normaliz\n");
      return -1;
    }
  if ((arg = analice_lista_exp (fp, mens_error))==NULL)
    {
      printf("Wrong expression:\n%s\n",mens_error);
      fclose (fp);
      return -1;
    }
  fclose(fp);
  normalizar_lista (&arg, &conj);
  fp = fopen ("normaliz", "w");
  if (fp==NULL)
    {
      printf("Couldn't write file normaliz\n");
      return -1;
    }
  FileOpen (fp, "normaliz");
  escriba_arch_ig (fp, conj, mensajes[79]);
  fclose (fp);
  imprimeArchivo("normaliz");  
  unlink("normaliz");
  
  return 0;
}



int testNormalization()
{
  printf("# Testing Normalization\n");    
  normaliza("blb_AND(N1,N2)");

  return 0;
}


int testTheorems()
{
  return 0;
}

int testPrototypes()
{
  return 0;
}


int testCopy(nodo_tad *nat,nodo_tad *TBool,nodo_tad *cola,nodo_tad *nogen)
{
  texpr merr;
  nodo_tad *t3;

  assert(nat!=NULL);
  assert(TBool!=NULL);
  assert(cola!=NULL);
  assert(nogen!=NULL);
  printf("# 2.2. Testing copying\n");
  merr[0]='\0';
  printf("# Basic Type\n");
  if (nuevoCopiaT("CNAT",nat,&t3,merr)!=0)
    printf(" %s",merr);
  else
    {
      imprimeT(t3,"Copy of NAT");
      eliminaT(&t3);
    }
  printf("# Extension\n");
  if (nuevoCopiaT("cnogen",nogen,&t3,merr)!=0)
    printf(" %s",merr);
  else
    {
      imprimeT(t3,"Copy of extension");
      eliminaT(&t3);
    }
  printf("# Type with parameters\n");
  if (nuevoCopiaT("Ccola",cola,&t3,merr)!=0)
    printf(" %s",merr);
  else
    {
      imprimeT(t3,"Copy of cola[x]");
      eliminaT(&t3);
    }
  return 0;
}



int testProduct(nodo_tad *nat,nodo_tad *TBool, nodo_tad *cola,
		    nodo_tad *nogen)
{
  nodo_tad *t3;
  texpr msg;

  assert(nat!=NULL);
  assert(TBool!=NULL);
  assert(cola!=NULL);
  assert(nogen!=NULL);
  printf("# 2.3. Testing product\n");
  msg[0]='\0';
  if (nuevoProdT("npn",nat,nat,&t3,msg)!=0)
    {
      printf("Couldn't make product NAT x NAT\n%s\n",msg);
    }
  else
    {
      imprimeT(t3,"Product NAT x NAT");
      eliminaT(&t3);
    }
  msg[0]='\0';
  if (nuevoProdT("npb",nat,TBool,&t3,msg)!=0)
    {
      printf("Couldn't make product NAT x TBool\n%s\n",msg);
    }
  else
    {
      imprimeT(t3,"Product NAT x TBool");
      eliminaT(&t3);
    }
  msg[0]='\0';
  if (nuevoProdT("bpn",TBool,nat,&t3,msg)!=0)
    {
      printf("Couldn't make product TBool x NAT\n%s\n",msg);
    }
  else
    {
      imprimeT(t3,"Product TBool x  NAT");
      eliminaT(&t3);
    }
  msg[0]='\0';
  if (nuevoProdT("cpn",cola,nat,&t3,msg)!=0)
    {
      printf("Couldn't make product cola x nat\n%s\n",msg);
    }
  else
    {
      imprimeT(t3,"Product cola x nat");
      eliminaT(&t3);
    }
  msg[0]='\0';
  if (nuevoProdT("npc",nat,cola,&t3,msg)!=0)
    {
      printf("Couldn't make product nat x cola\n%s\n",msg);
    }
  else
    {
      imprimeT(t3,"Product nat x cola");
      eliminaT(&t3);
    }
  msg[0]='\0';
  if (nuevoProdT("cpc",cola,cola,&t3,msg)!=0)
    {
      printf("Couldn't make product cola x cola\n%s\n",msg);
    }
  else
    {
      imprimeT(t3,"Product cola x cola");
      eliminaT(&t3);
    }
  msg[0]='\0';
  if (nuevoProdT("npg",nat,nogen,&t3,msg)!=0)
    {
      printf("Couldn't make product nat x nogen\n%s\n",msg);
    }
  else
    {
      imprimeT(t3,"Product nat x nogen1");
      eliminaT(&t3);
    }
  msg[0]='\0';
  if (nuevoProdT("gpn",nogen,nat,&t3,msg)!=0)
    {
      printf("Couldn't make product nogen x nat\n%s\n",msg);
    }
  else
    {
      imprimeT(t3,"Product nogen1 x nat");
      eliminaT(&t3);
    }
  return 0;
}


int testRestriction(nodo_tad *nat,nodo_tad *TBool, nodo_tad *cola,
		    nodo_tad *nogen)
{

  FILE *fp;
  nodo_tad *t3;
  texpr msg;

  assert(nat!=NULL);
  assert(TBool!=NULL);
  assert(cola!=NULL);
  assert(nogen!=NULL);
  printf("# 2.4. Testing restriction\n");
  fp=fopen("res1.mta","wt");
  if (fp==NULL)
    {
      printf("Couldn't write to res1.mta");
    }
  else
    {
      msg[0]='\0';
      if (nuevoResT("rnat",nat,"PAR",
		    "PAR(ZERO)=TRUE PAR(SUC(X))=NOT(PAR(X))",
		    fp,&t3,msg)!=0)
	{
	  printf("Couldn't make restriction of NAT to even\n%s\n",msg);
	}
      else
	{
	  imprimeT(t3,"Restriction of NAT to even (Generating res1.mta)");
	  eliminaT(&t3);
	}
      fclose(fp);
    }
  
  fp=fopen("res2.mta","wt");
  if (fp==NULL)
    {
      printf("Couldn't write to res2.mta");
    }
  else
    {
      msg[0]='\0';
      if (nuevoResT("rcola",cola,"COLAPAR",
		    "COLAPAR(vacia)=TRUE "
		    "COLAPAR(inserte(C1,X1))=NOT(COLAPAR(C1))",
		    fp,&t3,msg)!=0)
	{
	  printf("Couldn't make restriction of Cola to even\n%s\n",msg);
	}
      else
	{
	  imprimeT(t3,"Restriction of Cola to even (Generating res2.mta)");
	  eliminaT(&t3);
	}
      fclose(fp);
    }
  fp=fopen("res3.mta","wt");
  if (fp==NULL)
    {
      printf("Couldn't write to res3.mta");
    }
  else
    {
      msg[0]='\0';
      if (nuevoResT("rnogen",nogen,"TRIVIAL",
		    "TRIVIAL(X)=TRUE ",
		    fp,&t3,msg)!=0)
	{
	  printf("Couldn't make restriction of nogen1 to trivial\n%s\n",msg);
	}
      else
	{
	  imprimeT(t3,"Restriction of nogen to pred (Generated res3.mta)");
	  eliminaT(&t3);
	}
      fclose(fp);
    }
  return 0;
}


int testDirectSum(nodo_tad *nat,nodo_tad *TBool,nodo_tad *cola,nodo_tad *nogen)
{
  FILE *fp;
  nodo_tad *t3;
  texpr msg;

  assert(nat!=NULL);
  assert(TBool!=NULL);
  assert(cola!=NULL);
  assert(nogen!=NULL);
  printf("# 2.5. Testing Direct Sum\n");
  fp=fopen("sum1.mta","wt");
  if (fp==NULL)
    {
      printf("Couldn't write to sum1.mta");
    }
  else
    {
      msg[0]='\0';
      if (nuevoSumT("sum1",nat,TBool,
		    fp,&t3,msg)!=0)
	{
	  printf("Couldn't make direct sum of NAT with TBool\n%s\n",msg);
	  fclose(fp);
	}
      else
	{
	  fclose(fp);
	  imprimeArchivo("sum1.mta");
	  eliminaT(&t3);
	}
    }
  
  fp=fopen("sum2.mta","wt");
  if (fp==NULL)
    {
      printf("Couldn't write to sum2.mta");
    }
  else
    {
      msg[0]='\0';
      if (nuevoSumT("sum2",nat,cola,
		    fp,&t3,msg)!=0)
	{
	  printf("Couldn't make direct sum of NAT with cola\n%s\n",msg);
	  fclose(fp);
	}
      else
	{
	  fclose(fp);
	  imprimeArchivo("sum2.mta");
	  eliminaT(&t3);
	}
    }
  
  fp=fopen("sum3.mta","wt");
  if (fp==NULL)
    {
      printf("Couldn't write to sum3.mta");
    }
  else
    {
      msg[0]='\0';
      if (nuevoSumT("sum3",nogen,nat,
		    fp,&t3,msg)!=0)
	{
	  printf("Couldn't make direct sum of nogen with NAT\n%s\n",msg);
	  fclose(fp);
	}
      else
	{
	  fclose(fp);
	  imprimeArchivo("sum3.mta");
	  eliminaT(&t3);
	}
    }
  
  fp=fopen("sum4.mta","wt");
  if (fp==NULL)
    {
      printf("Couldn't write to sum4.mta");
    }
  else
    {
      msg[0]='\0';
      if (nuevoSumT("sum4",nat,nat,
		    fp,&t3,msg)!=0)
	{
	  printf("Couldn't make direct sum of NAT with NAT\n%s\n",msg);
	  fclose(fp);
	}
      else
	{
	  fclose(fp);
	  imprimeArchivo("sum4.mta");
	  eliminaT(&t3);
	}
    }
  return 0;
}


int testCompletation(nodo_tad *nat,nodo_tad *TBool,
		     nodo_tad *cola,nodo_tad *nogen)
{
  FILE *fp;
  nodo_tad *t3;
  texpr msg;

  assert(nat!=NULL);
  assert(TBool!=NULL);
  assert(cola!=NULL);
  assert(nogen!=NULL);
  printf("# 2.6. Testing Completation\n");
  fp=fopen("comp1.mta","wt");
  if (fp==NULL)
    {
      printf("Couldn't write to comp1.mta");
    }
  else
    {
      msg[0]='\0';
      if (nuevoComT("comp1",nat,"INF",
		    fp,&t3,msg)!=0)
	{
	  printf("Couldn't make completation of NAT with INF\n%s\n",msg);
	  fclose(fp);
	}
      else
	{
	  fclose(fp);
	  imprimeArchivo("comp1.mta");
	  eliminaT(&t3);
	}
    }

  fp=fopen("comp2.mta","wt");
  if (fp==NULL)
    {
      printf("Couldn't write to comp2.mta");
    }
  else
    {
      msg[0]='\0';
      if (nuevoComT("comp2",cola,"INF",
		    fp,&t3,msg)!=0)
	{
	  printf("Couldn't make completation of cola with INF\n%s\n",msg);
	  fclose(fp);
	}
      else
	{
	  fclose(fp);
	  imprimeArchivo("comp2.mta");
	  eliminaT(&t3);
	}
    }
  
  fp=fopen("comp3.mta","wt");
  if (fp==NULL)
    {
      printf("Couldn't write to comp3.mta");
    }
  else
    {
      msg[0]='\0';
      if (nuevoComT("comp3",nogen,"INF",
		    fp,&t3,msg)!=0)
	{
	  printf("Couldn't make completation of nogen with INF\n%s\n",msg);
	  fclose(fp);
	}
      else
	{
	  fclose(fp);
	  imprimeArchivo("comp3.mta");
	  eliminaT(&t3);
	}
    }
  
  fp=fopen("comp4.mta","wt");
  if (fp==NULL)
    {
      printf("Couldn't write to comp4.mta");
    }
  else
    {
      msg[0]='\0';
      if (nuevoComT("comp4",TBool,"ZERO",
		    fp,&t3,msg)!=0)
	{
	  printf("Couldn't make completation of TBool with ZERO\n%s\n",msg);
	  fclose(fp);
	}
      else
	{
	  fclose(fp);
	  imprimeArchivo("comp4.mta");
	  eliminaT(&t3);
	}
    }
  return 0;
}


int testVariation(nodo_tad *nat,nodo_tad *TBool,nodo_tad *cola, nodo_tad *nogen)
{
  FILE *fp;
  nodo_tad *t3;
  texpr msg;

  assert(nat!=NULL);
  assert(TBool!=NULL);
  assert(cola!=NULL);
  assert(nogen!=NULL);
  printf("# 2.7. Testing Variation\n");
  
  fp=fopen("var1.mta","wt");
  if (fp==NULL)
    {
      printf("Couldn't write to var1.mta");
    }
  else
    {
      msg[0]='\0';
      if (nuevoVarT("var1",TBool,nat,
		    fp,&t3,msg)!=0)
	{
	  printf("Couldn't make variation of TBool and NAT\n%s\n",msg);
	  fclose(fp);
	}
      else
	{
	  fclose(fp);
	  imprimeArchivo("var1.mta");
	  eliminaT(&t3);
	}
    }
  
  fp=fopen("var2.mta","wt");
  if (fp==NULL)
    {
      printf("Couldn't write to var2.mta");
    }
  else
    {
      msg[0]='\0';
      if (nuevoVarT("var2",TBool,TBool,
		    fp,&t3,msg)!=0)
	{
	  printf("Couldn't make variation of TBool and TBool\n%s\n",msg);
	  fclose(fp);
	}
      else
	{
	  fclose(fp);
	  imprimeArchivo("var2.mta");
	  eliminaT(&t3);
	}
    }
  
  fp=fopen("var3.mta","wt");
  if (fp==NULL)
    {
      printf("Couldn't write to var3.mta");
    }
  else
    {
      msg[0]='\0';
      if (nuevoVarT("var3",cola,TBool,
		    fp,&t3,msg)!=0)
	{
	  printf("Couldn't make variation of cola and TBool\n%s\n",msg);
	  fclose(fp);
	}
      else
	{
	  fclose(fp);
	  imprimeArchivo("var3.mta");
	  eliminaT(&t3);
	}
    }
  
  fp=fopen("var4.mta","wt");
  if (fp==NULL)
    {
      printf("Couldn't write to var4.mta");
    }
  else
    {
      msg[0]='\0';
      if (nuevoVarT("var4",nat,nogen,
		    fp,&t3,msg)!=0)
	{
	  printf("Couldn't make variation of nogen and NAT\n%s\n",msg);
	  fclose(fp);
	}
      else
	{
	  fclose(fp);
	  imprimeArchivo("var4.mta");
	  eliminaT(&t3);
	}
    }
  return 0;
}

int testInstance(nodo_tad *nat,nodo_tad *TBool,nodo_tad *cola,nodo_tad *nogen)
{
  nodo_tad *t;
  texpr merr;
  nodo_tad *t3;

  assert(nat!=NULL);
  assert(TBool!=NULL);
  assert(cola!=NULL);
  assert(nogen!=NULL);

  printf("# 2.1. Testing instancing\n");
  t=loadTextType("partype1.mta","Instance");
  printf("# Creating instance type\n");
  merr[0]='\0';
  if (nuevoInstanciacionT("inst1",t,"X",nat,&t3,merr)!=0)
    printf(" %s",merr);
  else
    {
      imprimeT(t3,"One parameter instanced with NAT");
      eliminaT(&t3);
    }
  printf("# Creating instance type\n");
  merr[0]='\0';
  if (nuevoInstanciacionT("inst2",t,"X",t,&t3,merr)!=0)
    printf(" %s",merr);
  else
    {
      imprimeT(t3,"One parameter instanced with cola");
      eliminaT(&t3);
    }
  eliminaT(&t);
  t=loadTextType("partype2.mta","Instance");
  merr[0]='\0';
  if (nuevoInstanciacionT("inst3",t,"Y",nat,&t3,merr)!=0)
    printf(" %s",merr);
  else
    {
      imprimeT(t3,"One of two parameters instanced with NAT");
      eliminaT(&t3);
    }
  eliminaT(&t);
  merr[0]='\0';
  if (nuevoInstanciacionT("inst3",cola,"X",nat,&t3,merr)!=0)
    printf(" %s",merr);
  else
    {
      imprimeT(t3,"cola[NAT]");
      eliminaT(&t3);
    }
  
  t=loadTextType("tins1.mta","Type with parameter and difficult EQ");
  merr[0]='\0';
  if (nuevoInstanciacionT("inst4",t,"X",nat,&t3,merr)!=0)
    printf(" %s",merr);
  else
    {
      imprimeT(t3,"Parameter of difficult type instanced with NAT");
      eliminaT(&t3);
    }
  eliminaT(&t);
  return 0;
}



int testAxioms()
{
  nodo_tad *t;

  printf("# 1.2. Testing axioms\n");
  t=loadTextType("taxiom1.mta","Axioms definitions");
  eliminaT(&t);
  t=loadTextType("taxiom2.mta","Axioms definitions");
  eliminaT(&t);
  t=loadTextType("taxiom3.mta","Axioms definitions");
  eliminaT(&t);
  t=    loadTextType("llaxiom1.mta","Axioms should be Left-Linear"); 
  eliminaT(&t);
  t=    loadTextType("llaxiom2.mta","Axioms should be Left-Linear"); 
  eliminaT(&t);
  t=    loadTextType("llaxiom3.mta","Axioms should be Left-Linear"); 
  eliminaT(&t);
  t=    loadTextType("llaxiom4.mta","Axioms should be Left-Linear"); 
  eliminaT(&t);
  t=loadTextType("caxiom1.mta","Total covering");
  eliminaT(&t);
  t=loadTextType("caxiom2.mta","Total covering");
  eliminaT(&t);
  t=loadTextType("caxiom3.mta","Total covering");
  eliminaT(&t);
  t=loadTextType("caxiom4.mta","Total covering");
  eliminaT(&t);
  t=loadTextType("aaxiom1.mta","Ambiguous definitions");
  eliminaT(&t);
  t=loadTextType("aaxiom2.mta","Ambiguous definitions");
  eliminaT(&t);
  t=loadTextType("aaxiom3.mta","Ambiguous definitions");
  eliminaT(&t);
  t=loadTextType("aaxiom4.mta","Ambiguous definitions, (this one should fail)");
  eliminaT(&t);
  t=loadTextType("ifaxiom1.mta","Axiom with if");
  eliminaT(&t);
  t=loadTextType("ifaxiom2.mta","Axiom with if");
  eliminaT(&t);
  t=loadTextType("ifaxiom3.mta","Axiom with if");
  eliminaT(&t);
  t=loadTextType("ifaxiom4.mta","Axiom with if");
  eliminaT(&t);
  t=loadTextType("eraxiom1.mta","ERROR not allowed at left side");
  eliminaT(&t);
  
  /* Chequear que todos tengan EQUAL ??? */
  t=loadTextType("eq1.mta","Types with generators have to have EQ");
  eliminaT(&t);
  t=loadTextType("eq2.mta","Types without generators can't have EQ");
  eliminaT(&t);
  t=loadTextType("eq3.mta","The signature of a valid EQ is unique");
  eliminaT(&t);
  t=loadTextType("eq4.mta","The signature of a valid EQ is unique");
  eliminaT(&t);
  
  
  /* Chequear nomenclatura.  Que una funcion definida en un TAD
     no sea: el nombre de una funcion definida en otro o el mismo TAD,
     el nombre de otro TAD, el nombre de un parametro del TAD 
     Una funcion no debe tener el mismo nombre del TAD (habra problemas
     en el momento de implementar)*/
  t=loadTextType("nomenc1.mta","Overloading is not allowed in initials");
  t=loadTextType("nomenc2.mta","Overloading is not allowed in initials");
  t=loadTextType("nomenc3.mta","Overloading is not allowed for selectors");
  t=loadTextType("nomenc4.mta","A function cannot have an ADT name");
  t=loadTextType("nomenc5.mta","A function cannot have an ADT name");
  t=loadTextType("nomenc6.mta","An ADT doesn't support overloading");
  t=loadTextType("nomenc7.mta","An ADT doesn't support overloading");
  t=loadTextType("nomenc8.mta","Functions names and parameters different");
  t=loadTextType("ADD.mta","An ADT can't have an already defined name");
  t=loadTextType("nomenc9.mta","An ADT parameter can't be a defined name");
  t=loadTextType("nomenc10.mta","All the ADT's parameter should differ");
  t=loadTextType("nomenc11.mta","The EQ_x names are reserved");
  t=loadTextType("nomenc12.mta","The EQ_x names are reserved");
  t=loadTextType("nomenc13.mta","The EQ_x names are reserved");
  
  /* Tipos con parametros */
  t=loadTextType("partype1.mta","Type with parameters");
  eliminaT(&t);
  t=loadTextType("partype2.mta","Type with two parameters");
  eliminaT(&t);
  t=loadTextType("partype3.mta","Type with parameters  that will produce an extension");
  eliminaT(&t);
  t=loadTextType("partype4.mta","Type with parameters without generators with domain invalid (itself)");
  eliminaT(&t);
  t=loadTextType("partype5.mta","Type with parameters without generators with equality");
  eliminaT(&t);
  t=loadTextType("partype6.mta","Type with parameters with generators and with equality");
  eliminaT(&t);
  t=loadTextType("partype7.mta","Type with parameters with generators and without equality");
  eliminaT(&t);
  return 0;
}


int testSignature()
{
  texpr nom;
  int k;
  nodo_tad *t;
  printf("# 1.1. Testing signatures\n");
  strcpy(nom,"domgen0.mta");
  for (k='1';k<'8';k++)
    {
      nom[6]=(char)k;
      t=loadTextType(nom,"Domain & Codomain of generators");
      eliminaT(&t);
    }
  t=loadTextType("domgen8.mta","Domain of initial can't be an extension");
  eliminaT(&t);
  t=loadTextType("domgen9.mta","Domain of selector can't be an extension");
  eliminaT(&t);
  t=loadTextType("domgen10.mta","Codomain of selector can't be an ext.");
  eliminaT(&t);
  t=loadTextType("domgen11.mta","Domain of a initial can't have the ADT");
  eliminaT(&t);
  t=loadTextType("domgen12.mta","Domain of a constructor has to have ADT");
  eliminaT(&t);
  strcpy(nom,"domsel0.mta");
  for (k='1';k<'9';k++)
    {
      nom[6]=(char)k;
      t=loadTextType(nom,"Domain & Codomain of selectors");
      eliminaT(&t);
    }
  t=loadTextType("domsel9.mta","The domain of a selector in an extension can't have the defined ADT");
  t=loadTextType("domsel10.mta","The codomain of a selector in an extension can't be the defined ADT");
  return 0;
}


int testGrammar()
{
  texpr nom;
  int k;
  nodo_tad *t;

  strcpy(nom,"empty0.mta");
  for (k='1';k<'7';k++)
    {
      nom[5]=(char)k;
      t=loadTextType(nom,"Empty case");
      eliminaT(&t);
    }
  t=loadTextType("noinit1.mta","No initializators");
  eliminaT(&t);
  t=loadTextType("nocons1.mta","No constructors");
  eliminaT(&t);
  t=loadTextType("nosel1.mta","No selectors");
  eliminaT(&t);
  t=loadTextType("noaxiom1.mta", "No axioms (no word AXIOMS)");
  eliminaT(&t);
  t=loadTextType("noaxiom2.mta", "No axioms (with word AXIOMS)");
  eliminaT(&t);
  t=loadTextType("noaxiom3.mta","Missing axiom in AXIOMS");
  eliminaT(&t);
  t=loadTextType("noaxiom4.mta","Missing axiom in SELECTORS");
  eliminaT(&t);
  t=loadTextType("daxiom1.mta","Different number of parameters in declaration and definition");
  eliminaT(&t);
  t=loadTextType("daxiom2.mta","Different codomain type in declaration and definition");
  eliminaT(&t);
  t=    loadTextType("daxiom3.mta","Different domain type in declaration and definition");
  eliminaT(&t);
  t=    loadTextType("ident1.mta","Identifiers");
  eliminaT(&t);
  t=    loadTextType("IDENT2.mta","Identifiers case sensitive");
  eliminaT(&t);
  t=loadTextType("ident3.mta","Identifiers case sensitive and length");
  eliminaT(&t);
  t=loadTextType("comen1.mta","Comentaries");
  eliminaT(&t);
  return 0;
}



int testRepresentations(nodo_tad *nat, nodo_tad *TBool,nodo_tad *cola,
			nodo_tad *nogen)
{
  FILE *fp;
  nodo_tad *lb,*bn,*nlb,*blb,*t;
  texpr msg,arcnam;

  assert(nat!=NULL);
  assert(TBool!=NULL);
  assert(cola!=NULL);
  assert(nogen!=NULL);
  printf("# Testing Inductive Representations\n");
  
  fp=fopen("rep1.mta","wt");
  if (fp==NULL)
    {
      printf("Couldn't write to rep1.mta");
      return -1;
    }
  msg[0]='\0';
  if (nuevaRep("rep1",TBool,nat,
	       fp,&bn,msg)!=0)
    {
      printf("Couldn't make representation of Bool with NAT\n%s\n",msg);
      fclose(fp);
      return -1;
    }
  fclose(fp);
  imprimeArchivo("rep1.mta");
  strcpy(arcnam,"rep1com.mta");
  fp=fopen(arcnam,"rt");
  if (fp==NULL)
    {
      printf("Couldn't read from %s",arcnam);
      return -1;
    }
  msg[0]='\0';
  if (cambiosRep(&bn,fp,msg)!=0)
    {
      printf("Couldn't change representation\n"
	     "%s\n",msg);
      fclose(fp);
      return -1;
    }
  fclose(fp);
  imprimeArchivo(arcnam);
  strcpy(arcnam,"conjrep1.thm");
  fp=fopen(arcnam,"wt");
  if (fp==NULL)
    {
      printf("Couldn't write to %s",arcnam);
      return -1;
    }
  msg[0]='\0';
  if (conjeturasRep(fp,bn,msg)!=0)
    {
      printf("Couldn't construct conjetures for representation\n"
	     "%s\n",msg);
      fclose(fp);
      return -1;
    }
  fclose(fp);
  imprimeArchivo(arcnam);
  unlink("conjrep1.thm");

  lb=loadTextType("lb.mta","List of Booleans\n");
  if (lb==NULL)
    {
      printf("File not found lb.mta");
      return -1;
    }
  if (adicElemLT(&tad,lb, ADIC_REP,msg)!=0)
    {
      printf("Couldn't add lb to environment\n%s",msg);
      return -1;
    }

  fp=fopen("nlb.mta","wt");
  if (fp==NULL)
    {
      printf("Couldn't write to nlb.mta");
      return -1;
    }
  msg[0]='\0';
  if (nuevaRep("nlb",nat,lb,
	       fp,&nlb,msg)!=0)
    {
      printf("Couldn't make representation of NAT with lb\n%s\n",msg);
      fclose(fp);
      return -1;
    }
  fclose(fp);
  imprimeArchivo("nlb.mta");
  strcpy(arcnam,"nlbcom.mta");
  fp=fopen(arcnam,"rt");
  if (fp==NULL)
    {
      printf("Couldn't read from %s",arcnam);
      return -1;
    }
  msg[0]='\0';
  if (cambiosRep(&nlb,fp,msg)!=0)
    {
      printf("Couldn't change representation\n"
	     "%s\n",msg);
      fclose(fp);
      return -1;
    }
  fclose(fp);
  imprimeArchivo(arcnam);
  if (adicElemLT(&tad,nlb, ADIC_REP,msg)!=0)
    {
      printf("Couldn't add nlb to environment\n%s",msg);
      return -1;
    }
  

  printf("\n# Composition of Representations\n");      

  msg[0]='\0';
  if (componeRep("blb",bn,nlb,&blb,msg)!=0)
    {
      printf("Couldn't make composition of bn with nlb\n");
      return -1;
    }
  fp=fopen("blb.mta","wt");
  if (fp==NULL)
    {
      printf("Couldn't write blb.mta\n");
      return -1;
    }
  creaArchTextoT(fp,blb);
  fclose(fp);
  t=loadTextType("blb.mta","Booleans represented with list of Booleans\n");

  blb->sig=tad;
  tad=blb;

  eliminaT(&t);

  return 0;
}





